#                            Package   : omniEvents
# eventc.py                  Created   : 16/11/2003
#                            Author    : Alex Tingle
#
#    Copyright (C) 2003 Alex Tingle.
#
#    This file is part of the omniEvents application.
#
#    omniEvents is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    omniEvents is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""Client to the event channel factory. Requests creation of an event
channel and registers it with the Naming service."""

import sys
import getopt
import CORBA
import naming
import CosNaming
import EventChannelAdmin
import CosLifeCycle
import CosEventChannelAdmin

def makeNVP(name,typecode,value):
  return CosLifeCycle.NameValuePair(name,CORBA.Any(typecode,value))


def main():
  #
  # Start orb.
  orb=CORBA.ORB_init(sys.argv, CORBA.ORB_ID)

  # Process Options
  channelName="EventChannel"
  channelKind="EventChannel"
  factoryName="EventChannelFactory"
  factoryKind="EventChannelFactory"
  pullRetryPeriod=1
  maxQueueLength=0
  maxEventsPerConsumer=0

  try:
    opts,args=getopt.getopt(sys.argv[1:],"n:k:N:K:r:m:q:h")
  except getopt.error:
    # print help information and exit:
    usage()
    sys.exit(-1)
  for option, optarg in opts:
    if option=='-n':
      channelName = optarg
    elif option=='-k':
      channelKind = optarg
    elif option=='-N':
      factoryName = optarg
    elif option=='-K':
      factoryKind = optarg
    elif option=='-r':
      pullRetryPeriod = int(optarg)
    elif option=='-m':
      maxEventsPerConsumer = int(optarg)
    elif option=='-q':
      maxQueueLength = int(optarg)
    elif option=='-h':
      usage()
      sys.exit(0)
    else:
      usage()
      sys.exit(-1)

  #
  # Get Name Service root context.
  rootContext = naming.getRootNamingContext(orb)

  #
  # Obtain reference to the Event Channel Factory implementation.
  name=[CosNaming.NameComponent(factoryName,factoryKind)]
  try:
      obj=rootContext.resolve(name)
      factory=obj._narrow(EventChannelAdmin.EventChannelFactory)
      if factory is None:
        sys.stderr.write("Failed to narrow Event Channel Factory reference.\n")
        sys.exit(1)
  except CORBA.COMM_FAILURE, ex:
    sys.stderr.write("Caught system exception COMM_FAILURE, unable to"+ \
      " contact the naming service.\n")
    sys.exit(1)
  except:
    sys.stderr.write('Cannot find event channel factory ! ["%s", "%s"]\n' \
      %(factoryName,factoryKind))
    sys.exit(1)

  # Check that the factory is of the right type
  key=[CosNaming.NameComponent("EventChannel","object interface")]
  try:
      if not factory.supports(key):
        sys.stderr.write('Factory does not support Event Channel Interface'+ \
          ' ! ["%s", "%s"]\n'%(factoryName,factoryKind))
        sys.exit(1)
  except:
    sys.stderr.write("Failure contacting Event Channel Factory\n")
    sys.exit(1)

  #
  # Create Event Channel Object.
  try:
      criteria=[
        makeNVP("PullRetryPeriod"     ,CORBA.TC_ulong,pullRetryPeriod),
        makeNVP("MaxEventsPerConsumer",CORBA.TC_ulong,maxEventsPerConsumer),
        makeNVP("MaxQueueLength"      ,CORBA.TC_ulong,maxQueueLength)]

      channelObj = factory.create_object(key,criteria)
      if channelObj is None:
        sys.stderr.write('Channel Factory returned nil reference.'+ \
          ' ! ["%s", "%s"]\n'%(channelName,channelKind))
        sys.exit(1)

      # Narrow object returned to an Event Channel
      channel=channelObj._narrow(CosEventChannelAdmin.EventChannel)
      if channel is None:
        sys.stderr.write('Failed to narrow Event Channel'+ \
          ' ! ["%s", "%s"]\n'%(channelName,channelKind))
        sys.exit(1)
  except CosLifeCycle.NoFactory, ex:
    sys.stderr.write('Failed to create Event Channel : '+ \
      'Interface not supported\n')
    sys.exit(1)
  except CosLifeCycle.CannotMeetCriteria, ex:
    sys.stderr.write('Failed to create Event Channel : Cannot meet Criteria\n')
    sys.exit(1)

  #
  # Register event channel with naming service
  name=[CosNaming.NameComponent(channelName,channelKind)]
  try:
      rootContext.bind(name,channel)
  except CosNaming.NamingContext.AlreadyBound, ex:
    rootContext.rebind(name,channel)
  except CORBA.COMM_FAILURE, ex:
    sys.stderr.write('Caught system exception COMM_FAILURE, unable to' + \
      ' contact the naming service.')
    sys.exit(1)
  except:
    sys.stderr.write('Cannot register event channel'+ \
      ' ! ["%s", "%s"]\n'%(channelName,channelKind))
    sys.exit(1)
#end main


def usage():
  sys.stderr.write("""
usage: eventc [-n name] [-k kind] [-N name] [-K kind] [-r sec] [-m events] [-q events] [-h]

         -n name specify channel name ["EventChannel"]
         -k kind specify channel kind ["EventChannel"]
         -N name specify factory name ["EventChannelFactory"]
         -K kind specify factory kind ["EventChannelFactory"]
         -r Pull retry Period (in seconds) [1 sec]
         -m Maximum Number of Events queued per Consumer [0 = unlimited]
         -q Maximum Number of Events queued by the channel [0 = unlimited]
         -h     display usage
""")


################################################################################
# If this file is executed directly, then we start here.
if(__name__=="__main__"):
  main()
