//                            Package   : omniEvents
// ConsumerAdmin.cc           Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "ConsumerAdmin.h"

#include "EventChannel.h"
#include "ProxyPushSupplier.h"
#include "ProxyPullSupplier.h"
#include "oep_types.h"

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

CosEventChannelAdmin::ProxyPushSupplier_ptr
ConsumerAdmin_i::obtain_push_supplier()
{
  if(!_pushSupplier)
      _pushSupplier=new ProxyPushSupplierManager(_poa,_queue);
  return _pushSupplier->createObject();
}


CosEventChannelAdmin::ProxyPullSupplier_ptr
ConsumerAdmin_i::obtain_pull_supplier()
{
  if(!_pullSupplier)
      _pullSupplier=new ProxyPullSupplierManager(_poa,_queue);
  return _pullSupplier->createObject();
}


ConsumerAdmin_i::ConsumerAdmin_i(
  const EventChannel_i&   channel,
  PortableServer::POA_ptr poa
)
: Servant(poa),
  _queue(channel.maxQueueLength()),
  _pushSupplier(NULL),
  _pullSupplier(NULL)
{
  activateObjectWithId("ConsumerAdmin");
}


ConsumerAdmin_i::ConsumerAdmin_i(
  const EventChannel_i&   channel,
  PortableServer::POA_ptr poa,
  const OEP_caps&         caps
)
: Servant(poa),
  _queue(channel.maxQueueLength()),
  _pushSupplier(NULL),
  _pullSupplier(NULL)
{
  // Build Push Supplier proxies
  if(!caps.getProxyPushSuppliers().empty())
  {
    _pushSupplier=new ProxyPushSupplierManager(_poa,_queue);
    _pushSupplier->reincarnate(caps.getProxyPushSuppliers());
  }

  // Build Pull Supplier proxies
  if(!caps.getProxyPullSuppliers().empty())
  {
    _pullSupplier=new ProxyPullSupplierManager(_poa,_queue);
    _pullSupplier->reincarnate(caps.getProxyPullSuppliers());
  }

  // Reactivate this servant.
  activateObjectWithId(caps.getKey());
}


ConsumerAdmin_i::~ConsumerAdmin_i()
{
  if(_pushSupplier)
  {
    delete _pushSupplier;
    _pushSupplier=NULL;
  }
  if(_pullSupplier)
  {
    delete _pullSupplier;
    _pullSupplier=NULL;
  }
}


void ConsumerAdmin_i::send(list<CORBA::Any*>& events)
{
  DB(20,"+ ConsumerAdmin_i::send() New events: "<<events.size());
  if(_pushSupplier)
  {
    {
      omni_mutex_lock l(_pushSupplier->_lock);
      _queue.append(events);
    }
    _pushSupplier->_condition.signal(); // Wake up the thread if it's asleep.
  }
  else
  {
    _queue.append(events);
  }
}


void ConsumerAdmin_i::discard(list<CORBA::Any*>& events)
{
  for(list<CORBA::Any*>::const_iterator i=events.begin();
      i!=events.end();
      ++i)
  {
    delete (*i);
  }
  events.clear();
}


void ConsumerAdmin_i::output(ostream& os)
{
  Output out(basicOutput(os,"consumerAdmin",2,this));
  if(_pushSupplier)
     _pushSupplier->output(os);
  if(_pullSupplier)
     _pullSupplier->output(os);
}


}; // end namespace OmniEvents
