//                            Package   : omniEvents
// EventChannel.cc            Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "EventChannel.h"
#include "ConsumerAdmin.h"
#include "SupplierAdmin.h"
#include "Orb.h"
#include "oep_types.h"
#include "defaults.h"

#include <list>
#include <assert.h>

namespace OmniEvents {

// CORBA interface methods
CosEventChannelAdmin::ConsumerAdmin_ptr EventChannel_i::for_consumers()
{
  if(!_consumerAdmin)
      _consumerAdmin=new ConsumerAdmin_i(*this,_poa);
  return _consumerAdmin->_this();
}


CosEventChannelAdmin::SupplierAdmin_ptr EventChannel_i::for_suppliers()
{
  if(!_supplierAdmin)
      _supplierAdmin=new SupplierAdmin_i(*this,_poa);
  return _supplierAdmin->_this();
}


void EventChannel_i::destroy()
{
  _shutdownRequested=true;
}


EventChannel_i::EventChannel_i(const char* channelName)
: Servant(PortableServer::POA::_nil()),
  _consumerAdmin(NULL),
  _supplierAdmin(NULL),
  _poaManager(),
  _shutdownRequested(false),
  _pullRetryPeriod(PULL_RETRY_PERIOD),
  _maxQueueLength(MAX_QUEUE_LENGTH)
{
  createPoa(channelName);
  activateObjectWithId("EventChannel");
}


EventChannel_i::EventChannel_i(const char* channelName, const OEP_ecps& ecps)
: Servant(PortableServer::POA::_nil()),
  _consumerAdmin(NULL),
  _supplierAdmin(NULL),
  _poaManager(),
  _shutdownRequested(false),
  _pullRetryPeriod(ecps.getPullRetryPeriod()),
  _maxQueueLength(ecps.getMaxQueueLength())
{
  createPoa(channelName);

  // Build Supplier Admin Object
  if(!ecps.getSupplierAdmins().empty())
  {
    OepSapsList::const_iterator sai =ecps.getSupplierAdmins().begin();
    _supplierAdmin=new SupplierAdmin_i(*this,_poa,**sai);
  }

  // Build Consumer Admin Object
  if(!ecps.getConsumerAdmins().empty())
  {
    OepCapsList::const_iterator cai =ecps.getConsumerAdmins().begin();
    _consumerAdmin=new ConsumerAdmin_i(*this,_poa,**cai);
  }

  activateObjectWithId("EventChannel");
}


EventChannel_i::~EventChannel_i()
{
  if(_consumerAdmin)
  {
    delete _consumerAdmin;
    _consumerAdmin=NULL;
  }
  if(_supplierAdmin)
  {
    delete _supplierAdmin;
    _supplierAdmin=NULL;
  }
}


void EventChannel_i::run(void* arg)
{
  try
  {

    _poaManager->activate();
    while(!_shutdownRequested)
    {
      if(_supplierAdmin)
      {
        _poaManager->hold_requests(CORBA::Boolean(1) /* wait_for_completion */);

        list<CORBA::Any*> events;
        _supplierAdmin->collect(events);
        if(_consumerAdmin)
            _consumerAdmin->send(events);
        else
            ConsumerAdmin_i::discard(events);
        assert(events.empty());

        _poaManager->activate();
      }
      omni_thread::sleep(0,10000000); // 0.1 second
    }
    _poaManager->deactivate(
      CORBA::Boolean(0) /* etherealize_objects */,
      CORBA::Boolean(1) /* wait_for_completion */
    );

  }
  catch(PortableServer::POAManager::AdapterInactive& ex)
  {
    cerr<<"POA deactivated from the outside."<<endl;
  }
  catch(...)
  {
    cerr<<"Unknown exception."<<endl;
  }

  // Thread now exits, and this object is deleted.
  // Contents are cleaned up by the destructor.
}


void EventChannel_i::output(ostream& os)
{
  CORBA::String_var poaName =_poa->the_name();

  os << "\teventChannel\n"
     << "\t{\n"
     << "\t\tKEY\t" << poaName.in() << "\n"
     << "\t\tMAXQUEUELENGTH\t" << _maxQueueLength << "\n"
  //??   << "\t\tMAXEVENTSPERCONSUMER\t" << _maxEventsPerConsumer << "\n"
     << "\t\tPULLRETRYPERIOD\t" << _pullRetryPeriod << "\n";
  if(_supplierAdmin)
     _supplierAdmin->output(os);
  if(_consumerAdmin)
     _consumerAdmin->output(os);
  os << "\t}\n";
}


void EventChannel_i::createPoa(const char* channelName)
{
  using namespace PortableServer;
  POA_ptr p=Orb::inst()._RootPOA.in();
  try
  {
    // POLICIES:
    //  Lifespan          =PERSISTENT             // we can persist
    //  Assignment        =USER_ID                // write our own oid
    //  Uniqueness        =[default] UNIQUE_ID    // one servant per object
    //  ImplicitActivation=[default] IMPLICIT_ACTIVATION // auto activation
    //  RequestProcessing =[default] USE_ACTIVE_OBJECT_MAP_ONLY
    //  ServantRetention  =[default] RETAIN       // stateless POA
    //  Thread            =SINGLE_THREAD_MODEL    // keep it simple

    CORBA::PolicyList policies;
    policies.length(3);
    policies[0]=p->create_lifespan_policy(PERSISTENT);
    policies[1]=p->create_id_assignment_policy(USER_ID);
    policies[2]=p->create_thread_policy(SINGLE_THREAD_MODEL);

    // Create a new POA (and new POAManager) for this channel.
    // The POAManager will be used for all of this channel's POAs.
    _poa=p->create_POA(channelName,POAManager::_nil(),policies);
    _poaManager=_poa->the_POAManager();
  }
  catch(POA::AdapterAlreadyExists& ex) // create_POA
  {
    cerr<<"POA::AdapterAlreadyExists"<<endl;
  }
  catch(POA::InvalidPolicy& ex) // create_POA
  {
    cerr<<"POA::InvalidPolicy: "<<ex.index<<endl;
  }
}


}; // end namespace OmniEvents

