//                            Package   : omniEvents
// ProxyManager.cc            Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "ProxyManager.h"
#include "Orb.h"

#include <string>
#include <assert.h>

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

//
//  ProxyManager
//

void
ProxyManager::etherealize(
  const PortableServer::ObjectId& oid,
  PortableServer::POA_ptr         adapter,
  PortableServer::Servant         serv,
  CORBA::Boolean                  cleanup_in_progress,
  CORBA::Boolean                  remaining_activations
)
{
  Proxy* narrowed =dynamic_cast<Proxy*>(serv);
  assert(narrowed);
  set<Proxy*>::iterator pos =_servants.find(narrowed);
  if(pos!=_servants.end())
      _servants.erase(pos);
  else
      DB(1,"\t\teh? - POA attempted to etherealize unknown servant.");
  delete narrowed;
}


void ProxyManager::reincarnate(const OepPrxyList& pdat)
{
  for(OepPrxyList::const_iterator iter=pdat.begin(); iter!=pdat.end(); iter++)
  {
    PortableServer::Servant serv =
      this->incarnate(PortableServer::ObjectId(),_managedPoa);
    Proxy* proxy =dynamic_cast<Proxy*>(serv);
    assert(proxy);
    try
    {
      proxy->reincarnate(**iter);
    }
    catch(CORBA::BAD_PARAM& ex)
    {
      // This will happen when (*iter)->getIor() fails to narrow.
      DB(1,"Failed to reincarnate proxy: "<<(*iter)->getKey());
      _servants.erase(proxy);
      delete proxy;
    }
  }
}


void ProxyManager::output(ostream& os)
{
  for(set<Proxy*>::iterator i =_servants.begin(); i!=_servants.end(); ++i)
  {
    (*i)->output(os);
  }
}


ProxyManager::ProxyManager(PortableServer::POA_ptr p, const char* name)
: Servant(p),
  _servants(),
  _managedPoa(PortableServer::POA::_nil())
{
  using namespace PortableServer;
  try
  {  
    // POLICIES:
    //  Lifespan          =PERSISTENT             // we can persist
    //  Assignment        =USER_ID                // write our own oid
    //  Uniqueness        =[default] UNIQUE_ID    // one servant per object
    //  ImplicitActivation=NO_IMPLICIT_ACTIVATION // disable auto activation
    //  RequestProcessing =USE_SERVANT_MANAGER
    //  ServantRetention  =[default] RETAIN
    //  Thread            =SINGLE_THREAD_MODEL    // keep it simple

    CORBA::PolicyList policies;
    policies.length(5);
    policies[0]=p->create_lifespan_policy(PERSISTENT);
    policies[1]=p->create_id_assignment_policy(USER_ID);
    policies[2]=p->create_implicit_activation_policy(NO_IMPLICIT_ACTIVATION);
    policies[3]=p->create_request_processing_policy(USE_SERVANT_MANAGER);
    policies[4]=p->create_thread_policy(SINGLE_THREAD_MODEL);

    // Create a POA for this proxy type in this channel.
    CORBA::String_var parentName    =p->the_name();
    string            poaName       =string(parentName.in())+"."+name;
    POAManager_var    parentManager =p->the_POAManager();
    _managedPoa=p->create_POA(poaName.c_str(),parentManager.in(),policies);
  }
  catch(POA::AdapterAlreadyExists& ex) // create_POA
  {
    cerr<<"POA::AdapterAlreadyExists"<<endl;
  }
  catch(POA::InvalidPolicy& ex) // create_POA
  {
    cerr<<"POA::InvalidPolicy: "<<ex.index<<endl;
  }
  string oidStr =string(name)+"Manager";
  activateObjectWithId(oidStr.c_str());
  _managedPoa->set_servant_manager(_this());
}


ProxyManager::~ProxyManager()
{
  // pass
}


//
// Proxy
//


Proxy::~Proxy()
{
  if(!CORBA::is_nil(_req))
  {
    Orb::inst().orphanRequest(_req._retn());
    _req=CORBA::Request::_nil();
  }
}

bool Proxy::gotException()
{
  CORBA::Environment_var env=_req->env();
  if(!CORBA::is_nil(env) && env->exception())
  {
    CORBA::Exception* ex =env->exception(); // No need to free exception.
    DB(1,"Proxy got exception: "<<ex->_name()
      <<" while calling "<<_req->operation()<<"().");
    _req=CORBA::Request::_nil();
    deactivateObject();
    return true;
  }
  return false;
}

Proxy::Proxy(PortableServer::POA_ptr poa)
: Servant(poa),
  _req(CORBA::Request::_nil())
{
  // pass
}


}; // end namespace OmniEvents
