//                            Package   : omniEvents
// ProxyPullConsumer.cc       Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "ProxyPullConsumer.h"
#include "Orb.h"
#include "omniEventsLog.h"

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

//
//  ProxyPullConsumerManager
//

PortableServer::Servant
ProxyPullConsumerManager::incarnate(
  const PortableServer::ObjectId& oid,
  PortableServer::POA_ptr         poa
)
{
  // ?? Check that OID is valid here.
  ProxyPullConsumer_i* result =new ProxyPullConsumer_i(_managedPoa,_queue);
  _servants.insert(result);
  return result;
}

ProxyPullConsumerManager::ProxyPullConsumerManager(
  PortableServer::POA_ptr parentPoa,
  list<CORBA::Any*>&      q
)
: ProxyManager(parentPoa,"ProxyPullConsumer"),
  _queue(q)
{
  // pass
}

CosEventChannelAdmin::ProxyPullConsumer_ptr
ProxyPullConsumerManager::createObject()
{  
  return createNarrowedReference<CosEventChannelAdmin::ProxyPullConsumer>(
           _managedPoa,
           CosEventChannelAdmin::_tc_ProxyPullConsumer->id()
         );
}



void ProxyPullConsumerManager::trigger()
{
  // Trigger each servant in turn.
  for(set<Proxy*>::iterator i =_servants.begin(); i!=_servants.end(); ++i)
  {
    ProxyPullConsumer_i* proxy=static_cast<ProxyPullConsumer_i*>(*i);
    proxy->trigger();
  }
}


//
//  ProxyPullConsumer_i
//

// CORBA interface methods

void ProxyPullConsumer_i::connect_pull_supplier(
  CosEventComm::PullSupplier_ptr pullSupplier
)
{
  try
  {
    if(CORBA::is_nil(pullSupplier))
        throw CORBA::BAD_PARAM();
    if(!CORBA::is_nil(_target) || !CORBA::is_nil(_req))
        throw CosEventChannelAdmin::AlreadyConnected();
    _target=CosEventComm::PullSupplier::_duplicate(pullSupplier);

    omniEventsLog::persist();
  }
  catch(...) // ?? AlreadyConnected -> deactivate. DOS attack!!
  {
    _target=CosEventComm::PullSupplier::_nil();
    deactivateObject(); //??  <<===== Get rid of this.
    throw;
  }
}

void ProxyPullConsumer_i::disconnect_pull_consumer()
{
  DB(5,"ProxyPullConsumer_i::disconnect_pull_consumer()");
  deactivateObject();
  if(CORBA::is_nil(_target))
  {
    throw CORBA::BAD_INV_ORDER(
      omni::BAD_INV_ORDER_RequestUsedMoreThanOnce,
      CORBA::COMPLETED_YES
    );
  }
  else
  {
    CORBA::Request_var req=_target->_request("disconnect_pull_supplier");
    req->send_deferred();
    Orb::inst().orphanRequest(req._retn());
    _target=CosEventComm::PullSupplier::_nil();
  }
}

//

ProxyPullConsumer_i::ProxyPullConsumer_i(
  PortableServer::POA_ptr poa,
  list<CORBA::Any*>&      q
)
: Proxy(poa),
  _target(CosEventComm::PullSupplier::_nil()),
  _queue(q)
{
  _exceptionCount[Pull]=0;
  _exceptionCount[TryPull]=0;
}

void ProxyPullConsumer_i::trigger()
{
  // PSEUDOCODE:
  // If an exception has arrived, increment the exception count for that
  //   operation (pull or try_pull), and switch to the other operation.
  // If both operations' exception counts are >1, then deactivate this object.
  // If a method has returned, then desice whether we have an event (hasEvent).
  //   If so, the read the event and push it onto the queue.
  // If there is no request in progress, send a new request to the current
  //   operation (pull or try_pull).

  // Prefer 'pull' method calls.
  CORBA::String_var opname ="pull";

  if(!CORBA::is_nil(_req) && _req->poll_response()) ////// RESPONSE HAS ARRIVED
  {
    opname=CORBA::string_dup(_req->operation());

    CORBA::Environment_var env=_req->env();
    if(!CORBA::is_nil(env) && env->exception()) //////////// EXCEPTION OCCURRED
    {
      CORBA::Exception* ex =env->exception(); // No need to free exception.
      DB(1,"ProxyPullConsumer got exception: "<<ex->_name()<<", op:"<<opname);
      _req=CORBA::Request::_nil();
      if(0==strcmp("pull",opname))
      {
        ++(_exceptionCount[Pull]);
        opname="try_pull"; // Try something else next time.
      }
      else if(0==strcmp("try_pull",opname))
      {
        ++(_exceptionCount[TryPull]);
        opname="pull"; // Try something else next time.
      }
      else
          DB(1,"Ignoring unrecognised response. operation:"<<opname);
      if(_exceptionCount[Pull]>=2 && _exceptionCount[TryPull]>=2)
      {
        _target=CosEventComm::PullSupplier::_nil(); // disconnected
        deactivateObject();
        return;
      }
    }
    else  //////////////////////////////////////////////// METHOD CALL RETURNED
    {
      // Do we have an event?
      bool hasEvent=false;
      if(0==strcmp("pull",opname))
      {
        _exceptionCount[Pull]=0;
        hasEvent=true;
      }
      else if(0==strcmp("try_pull",opname))
      {
        _exceptionCount[TryPull]=0;
        CORBA::NVList_var args=_req->arguments();
        if(args->count()==1)
        {
          CORBA::NamedValue_var hasEventArg=args->item(0);
          if(0==strcmp(hasEventArg->name(),"has_event"))
          {
            CORBA::Any* a =hasEventArg->value();
            CORBA::Boolean b;
            CORBA::Any::to_boolean tb(b); //MS VC++6 is on drugs!
            hasEvent=(((*a)>>=tb) && b);
          }
        }
      }
      // Pick up an event, if we have one.
      if(hasEvent)
      {
        CORBA::Any* event =new CORBA::Any();
        _req->return_value() >>= (*event);
        _queue.push_back(event);
      }
    }
    _req=CORBA::Request::_nil();
  }
  ////////////////////////////////////////////////// SEND OUT A NEW METHOD CALL
  if(CORBA::is_nil(_req) && !CORBA::is_nil(_target))
  {
    _req=_target->_request(opname);
    _req->set_return_type(CORBA::_tc_any);
    if(0==strcmp("try_pull",opname))
        _req->add_out_arg("has_event")<<=CORBA::Any::from_boolean(1);
    _req->send_deferred();
  }
}

void ProxyPullConsumer_i::reincarnate(const OEP_prxy& prxy)
{
  CosEventComm::PullSupplier_var pullSupplier =
    string_to_<CosEventComm::PullSupplier>(prxy.getIor());
  // Do not activate until we know that we have read a valid target.
  activateObjectWithId(prxy.getKey());
  connect_pull_supplier(pullSupplier.in());
}

void ProxyPullConsumer_i::output(ostream& os)
{
  Output out =basicOutput(os,"pxyPullConsumer",3,this,_target.in());
}

}; // end namespace OmniEvents
