//                            Package   : omniEvents
// ProxyPushConsumer.cc       Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "ProxyPushConsumer.h"
#include "Orb.h"
#include "omniEventsLog.h"

#include <assert.h>

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

void ProxyPushConsumer_i::connect_push_supplier(
  CosEventComm::PushSupplier_ptr pushSupplier)
{
  // pushSupplier is permitted to be nil.
  if(CORBA::is_nil(pushSupplier))
      return;

  ObjectId oid =currentObjectId();
  Connections_t::iterator pos =_connections.find(oid);

  if(pos!=_connections.end())
      throw CosEventChannelAdmin::AlreadyConnected();

  _connections.insert(Connections_t::value_type(
      oid,
      CosEventComm::PushSupplier::_duplicate(pushSupplier)
  ));

  omniEventsLog::persist();
}


void ProxyPushConsumer_i::disconnect_push_consumer()
{
  DB(5,"ProxyPushConsumer_i::disconnect_push_consumer()");
  ObjectId oid =currentObjectId();
  Connections_t::iterator pos =_connections.find(oid);

  if(pos!=_connections.end())
  {
    CORBA::Request_var req =
      pos->second->_request("disconnect_push_supplier");
    req->send_oneway();
    _connections.erase(pos);
  }
}


void ProxyPushConsumer_i::push(const CORBA::Any& event)
{
  _queue.push_back(new CORBA::Any(event));
}


ProxyPushConsumer_i::ProxyPushConsumer_i(
  PortableServer::POA_ptr p,
  list<CORBA::Any*>& q
)
: Servant(PortableServer::POA::_nil()),
  _connections(),
  _queue(q)
{
  using namespace PortableServer;
  try
  {  
    // POLICIES:
    //  Lifespan          =PERSISTENT             // we can persist
    //  Assignment        =USER_ID                // write our own oid
    //  Uniqueness        =MULTIPLE_ID            // only one servant
    //  ImplicitActivation=NO_IMPLICIT_ACTIVATION // disable auto activation
    //  RequestProcessing =USE_DEFAULT_SERVANT    // only one servant
    //  ServantRetention  =NON_RETAIN             // stateless POA
    //  Thread            =SINGLE_THREAD_MODEL    // keep it simple

    CORBA::PolicyList policies;
    policies.length(7);
    policies[0]=p->create_lifespan_policy(PERSISTENT);
    policies[1]=p->create_id_assignment_policy(USER_ID);
    policies[2]=p->create_id_uniqueness_policy(MULTIPLE_ID);
    policies[3]=p->create_implicit_activation_policy(NO_IMPLICIT_ACTIVATION);
    policies[4]=p->create_request_processing_policy(USE_DEFAULT_SERVANT);
    policies[5]=p->create_servant_retention_policy(NON_RETAIN);
    policies[6]=p->create_thread_policy(SINGLE_THREAD_MODEL);

    // Create a POA for this proxy type in this channel.
    CORBA::String_var parentName  =p->the_name();
    string          poaName       =string(parentName.in())+".ProxyPushConsumer";
    POAManager_var  parentManager =p->the_POAManager();
    _poa=p->create_POA(poaName.c_str(),parentManager.in(),policies);
  }
  catch(POA::AdapterAlreadyExists& ex) // create_POA
  {
    cerr<<"POA::AdapterAlreadyExists"<<endl;
  }
  catch(POA::InvalidPolicy& ex) // create_POA
  {
    cerr<<"POA::InvalidPolicy: "<<ex.index<<endl;
  }
  // This object is the POA's default servant.
  _poa->set_servant(this);
}


ProxyPushConsumer_i::~ProxyPushConsumer_i()
{
  // pass (_poa is destroyed by its parent.)
}


CosEventChannelAdmin::ProxyPushConsumer_ptr
ProxyPushConsumer_i::createObject()
{
  return createNarrowedReference<CosEventChannelAdmin::ProxyPushConsumer>(
           _poa,
           CosEventChannelAdmin::_tc_ProxyPushConsumer->id()
         );
}


void ProxyPushConsumer_i::reincarnate(
  const char* thisOid,
  const char* supplierIOR
)
{
  ObjectId oid(thisOid);

  Connections_t::iterator pos =_connections.find(oid);
  if(pos!=_connections.end())
      throw CosEventChannelAdmin::AlreadyConnected();

  CosEventComm::PushSupplier_var pushSupplier;
  try
  {
    CORBA::Object_ptr obj =Orb::inst()._orb->string_to_object(supplierIOR);
    pushSupplier=CosEventComm::PushSupplier::_narrow(obj);
  }
  catch(...)//??
  {
  }

  if(CORBA::is_nil(pushSupplier))
      throw CORBA::BAD_PARAM();

  _connections.insert(Connections_t::value_type(
      oid,
      pushSupplier._retn()
  ));
}


void ProxyPushConsumer_i::output(ostream& os) const
{
  for(Connections_t::const_iterator i=_connections.begin();
      i!=_connections.end();
      ++i)
  {
    os << "\t\t\tpxyPushConsumer\n"
       << "\t\t\t{\n"
       << "\t\t\t\tKEY\t" << i->first._str << "\n";
    if(!CORBA::is_nil(i->second.in()))
    {
      CORBA::String_var iorstr;
      iorstr = Orb::inst()._orb->object_to_string(i->second.in());
      os << "\t\t\t\tIOR\t" << iorstr << "\n";
    }
    os << "\t\t\t}\n";
  }
}


ProxyPushConsumer_i::ObjectId
ProxyPushConsumer_i::currentObjectId() const
{
  ProxyPushConsumer_i::ObjectId result;
  try
  {
    using namespace PortableServer;
    result._oid=Orb::inst()._POACurrent->get_object_id();
    CORBA::String_var oidStr =ObjectId_to_string(result._oid.in());
    result._str=string(oidStr.in());
  }
  catch(PortableServer::Current::NoContext ex) // get_object_id()
  {
    cerr<<"No context!!"<<endl;
  }
  catch(CORBA::BAD_PARAM ex) // ObjectId_to_string()
  {
    // Should never get here in omniORB, because ObjectID is a char*.
    assert(0);
  }
  return result;
}


//
//  ProxyPushConsumer_i::ObjectId
//

ProxyPushConsumer_i::ObjectId::ObjectId(const char* str)
: _str(str)
{
  _oid=PortableServer::string_to_ObjectId(_str.c_str());
}


}; // end namespace OmniEvents
