//                            Package   : omniEvents
// ProxyPushSupplier.cc       Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "ProxyPushSupplier.h"
#include "Orb.h"
#include "omniEventsLog.h"

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

//
//  ProxyPushSupplierManager
//

PortableServer::Servant
ProxyPushSupplierManager::incarnate(
  const PortableServer::ObjectId& oid,
  PortableServer::POA_ptr         poa
)
{
  // ?? Check that OID is valid here.
  ProxyPushSupplier_i* result =new ProxyPushSupplier_i(_managedPoa,_queue);
  {
    omni_mutex_lock l(_lock);
    _servants.insert(result);
  }
  _condition.signal(); // Wake up the thread if it's asleep.
  return result;
}

void
ProxyPushSupplierManager::etherealize(
  const PortableServer::ObjectId& oid,
  PortableServer::POA_ptr         adapter,
  PortableServer::Servant         serv,
  CORBA::Boolean                  cleanup_in_progress,
  CORBA::Boolean                  remaining_activations
)
{
  omni_mutex_lock l(_lock);
  ProxyManager::etherealize(oid,adapter,serv,
    cleanup_in_progress,remaining_activations);
}


ProxyPushSupplierManager::ProxyPushSupplierManager(
  PortableServer::POA_ptr parentPoa,
  EventQueue& q
)
: ProxyManager(parentPoa,"ProxyPushSupplier"),
  _queue(q),
  _lock(),_conditionLock(),_condition(&_conditionLock),
  _shutdownRequested(false)
{
  start_undetached();
}

CosEventChannelAdmin::ProxyPushSupplier_ptr
ProxyPushSupplierManager::createObject()
{  
  return createNarrowedReference<CosEventChannelAdmin::ProxyPushSupplier>(
           _managedPoa,
           CosEventChannelAdmin::_tc_ProxyPushSupplier->id()
         );
}

void*
ProxyPushSupplierManager::run_undetached(void*)
{
  omni_mutex_lock cl(_conditionLock);
  while(true)
  {
    bool moreWorkToDo=false;
    {
      omni_mutex_lock l(_lock); // Block here when paused.
      if(_shutdownRequested)
          break;

      // Trigger each servant in turn.
      for(set<Proxy*>::iterator i =_servants.begin(); i!=_servants.end(); ++i)
      {
        ProxyPushSupplier_i* proxy=static_cast<ProxyPushSupplier_i*>(*i);
        if(proxy->trigger())
            moreWorkToDo=true;
      }
    }
    
    // Wait
    if(moreWorkToDo)
        omni_thread::yield();
    else
        _condition.wait(); // wait until something happens.
  }
  return NULL;
}


//
//  ProxyPushSupplier_i
//

void ProxyPushSupplier_i::connect_push_consumer(
  CosEventComm::PushConsumer_ptr pushConsumer)
{
  try
  {
    if(CORBA::is_nil(pushConsumer))
        throw CORBA::BAD_PARAM();
    if(!CORBA::is_nil(_target) || !CORBA::is_nil(_req))
        throw CosEventChannelAdmin::AlreadyConnected();
    _target=CosEventComm::PushConsumer::_duplicate(pushConsumer);

    omniEventsLog::persist();
  }
  catch(...)
  {
    _target=CosEventComm::PushConsumer::_nil(); // disconnected.
    deactivateObject();
    throw;
  }
}


void ProxyPushSupplier_i::disconnect_push_supplier()
{
  DB(5,"ProxyPushSupplier_i::disconnect_push_supplier()");
  deactivateObject();
  if(CORBA::is_nil(_target))
  {
    throw CORBA::BAD_INV_ORDER(
      omni::BAD_INV_ORDER_RequestUsedMoreThanOnce,
      CORBA::COMPLETED_YES
    );
  }
  else
  {
    CORBA::Request_var req=_target->_request("disconnect_push_consumer");
    req->send_deferred();
    Orb::inst().orphanRequest(req._retn());
    _target=CosEventComm::PushConsumer::_nil();
  }
}


ProxyPushSupplier_i::ProxyPushSupplier_i(
  PortableServer::POA_ptr poa,
  EventQueue&             q
)
: Proxy(poa),
  EventQueue::Reader(q),
  _target(CosEventComm::PushConsumer::_nil())
{
  // pass
}


bool ProxyPushSupplier_i::trigger()
{
  if(!CORBA::is_nil(_req) && _req->poll_response()) // response has arrived
  {
    CORBA::Environment_var env=_req->env();
    if(!CORBA::is_nil(env) && env->exception())
    {
      CORBA::Exception* ex =env->exception(); // No need to free exception.
      DB(1,"ProxyPushSupplier got exception: "<<ex->_name());
      _req=CORBA::Request::_nil();
      _target=CosEventComm::PushConsumer::_nil(); // disconnected.
      deactivateObject();
      return false; // No more work to do
    }
    _req=CORBA::Request::_nil();
  }
  if(CORBA::is_nil(_req) && !CORBA::is_nil(_target) && moreEvents())
  {
    _req=_target->_request("push");
    _req->add_in_arg() <<= *(nextEvent());
    _req->send_deferred();
  }
  return !CORBA::is_nil(_req); // More work to do, if _req NOT nil.
}


void ProxyPushSupplier_i::reincarnate(const OEP_prxy& prxy)
{
  CosEventComm::PushConsumer_var pushConsumer =
    string_to_<CosEventComm::PushConsumer>(prxy.getIor());
  // Do not activate until we know that we have read a valid target.
  activateObjectWithId(prxy.getKey());
  connect_push_consumer(pushConsumer.in());
}


void ProxyPushSupplier_i::output(ostream &os)
{
  Output out =basicOutput(os,"pxyPushSupplier",3,this,_target.in());
}


}; // end namespace OmniEvents
