//                            Package   : omniEvents
// Servant.cc                 Created   : 2003/12/04
//                            Author    : Alex Tingle
//
//    Copyright (C) 2003 Alex Tingle.
//
//    This file is part of the omniEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Servant.h"
#include "Orb.h"

#ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h> // getpid
#endif

#ifdef HAVE_UNISTD_H
#  include <unistd.h>    // getpid
#elif defined(HAVE_PROCESS_H)
# include <process.h>
#endif

#include <stdio.h>     // sprintf
#include <assert.h>

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

namespace OmniEvents {

CORBA::Object_ptr createReference(
  PortableServer::POA_ptr poa,          // POA to own new object
  const char*             repositoryId  // e.g. _tc_ProxyPushSupplier->id()
)
{
  CORBA::String_var oidStr =newUniqueId();

  PortableServer::ObjectId_var oid =
    PortableServer::string_to_ObjectId(oidStr.in());

  CORBA::Object_var obj =
    poa->create_reference_with_id(oid.in(),repositoryId);

  assert(!CORBA::is_nil(obj));
  return obj._retn();
}

char* newUniqueId()
{
  static long  count=0;
  static omni_mutex  mutex;
  int  mypid =getpid(); // MS VC++6 doesn't have type pid_t!
  unsigned long  sec,nsec;
  omni_thread::get_time(&sec,&nsec); // More portable than just time().
  char buf[128];
  {
    omni_mutex_lock l(mutex);
    sprintf(buf,"%x.%x.%x",mypid,sec,++count);
  }
  return CORBA::string_dup(buf);
}


//
//  Servant
//

Servant::Servant(PortableServer::POA_ptr poa)
: _poa(PortableServer::POA::_duplicate(poa))
{
  // pass
}


Servant::~Servant()
{
  // pass
}


PortableServer::POA_ptr Servant::_default_POA()
{
  return PortableServer::POA::_duplicate(_poa.in());
}


void Servant::activateObjectWithId(const char* oidStr)
{
  using namespace PortableServer;
  CORBA::String_var poaName =_poa->the_name();
  DB(5,"Activating object "<<poaName.in()<<"/"<<oidStr);
  try
  {
    ObjectId_var oid =string_to_ObjectId(oidStr);
    _poa->activate_object_with_id(oid.in(),this);
  }
  catch(CORBA::BAD_PARAM& ex)
  {
    cerr<<"Can't activate "<<oidStr<<".\n"
      "BAD_PARAM: "<<ex.NP_minorString()<<endl;
  }
  catch(POA::ServantAlreadyActive& ex)
  {
    cerr<<"Can't activate "<<oidStr<<".\nServant is already active."<<endl;
  }
  catch(POA::ObjectAlreadyActive& ex)
  {
    cerr<<"Can't activate "<<oidStr<<".\nObject is already active."<<endl;
  }
  catch(POA::WrongPolicy& ex)
  {
    cerr<<"Can't activate "<<oidStr<<".\nPOA '"<<poaName.in()
        <<"' has wrong policy for activate_object_with_id()."<<endl;
    exit(1); // Programming error - so quit.
  }
}


void Servant::deactivateObject()
{
  using namespace PortableServer;
  CORBA::String_var poaName =_poa->the_name();

  ObjectId_var oid;
  try
  {
    oid=_poa->servant_to_id(this);
  }
  catch(POA::ServantNotActive& ex)
  {
    cerr<<"Can't deactivate servant.\nPOA '"<<poaName.in()
        <<"' says it is not active."<<endl;
  }
  catch(POA::WrongPolicy& ex)
  {
    cerr<<"Can't deactivate servant.\nPOA '"<<poaName.in()
        <<"' has wrong policy for servant_to_id()."<<endl;
    exit(1); // Programming error - so quit.
  }

  CORBA::String_var oidStr;
  try
  {
    oidStr=ObjectId_to_string(oid.in());
  }
  catch(CORBA::BAD_PARAM& ex)
  {
    cerr<<"Can't deactivate servant. ObjectId looks bad.\n"
      "BAD_PARAM: "<<ex.NP_minorString()<<endl;
  }

  try
  {
    DB(5,"Deactivating object "<<poaName<<"/"<<oidStr.in());
    _poa->deactivate_object(oid.in());
  }
  catch(POA::ObjectNotActive& ex)
  {
    cerr<<"Can't deactivate "<<oidStr<<".\nObject is not active."<<endl;
  }
  catch(POA::WrongPolicy& ex)
  {
    cerr<<"Can't deactivate "<<oidStr<<".\nPOA '"<<poaName.in()
        <<"' has wrong policy for deactivate_object()."<<endl;
    exit(1); // Programming error - so quit.
  }
}

Servant::Output Servant::basicOutput(
  ostream&                     os,
  const char*                  name,
  int                          indent,
  PortableServer::ServantBase* servant,
  CORBA::Object_ptr            target
)
{
  string indentStr(indent,'\t');
  Output result(os,indentStr);

  PortableServer::ObjectId_var oid=_poa->servant_to_id(servant);
  CORBA::String_var oidStr =PortableServer::ObjectId_to_string(oid.in());

  os<<indentStr<<name<<"\n"
    <<indentStr<<"{\n"
    <<indentStr<<"\tKEY\t" << oidStr.in() << "\n";

  if(!CORBA::is_nil(target))
  {
    os<<indentStr<<"\tIOR\t";
    CORBA::String_var iorstr =Orb::inst()._orb->object_to_string(target);
    os << iorstr.in() << "\n";
  }
  return result;
}

}; // end namespace OmniEvents
