// -*- Mode: C++; -*-
//                            Package   : omniEvents
//  omniEventsLog.cc          Created   : 1/10/99
//                            Author    : Paul Nader (pwn)
//
//    Copyright (C) 1998 Paul Nader.
//
//    This file is part of the omniOEvents application.
//
//    omniEvents is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    omniEvents is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Description:
//	

/*
  $Log: omniEventsLog.cc,v $
  Revision 1.4  2003/11/14 14:05:21  alextingle
  New output() members functions. Eliminates the need for friend ostream
  functions that are problematic on earlier versions of Microsoft
  VisualC++. Improved helpfulness of usage and error messages.

  Revision 1.3  2003/11/03 22:41:00  alextingle
  Oops! Removed excess log comments.

  Revision 1.2  2003/11/03 22:38:39  alextingle
  Removed many platform specific switches, alas many remain. Now uses
    autoconf,config.h wherever possible.
  Removed local definition of strdup() - omniEvents.cc manages to get
    along without it, so it can't be needed here.
  Moved convoluted code to obtain hostname into its own header file:
    gethostname.h
  Moved code that constructs logfile names into its own private method,
    omniEventsLog::initializeFileNames()
  Added explicit initialisation for all `class omniEventsLog' members.
  Moved code that opens file stream into its own private method,
    omniEventsLog::openOfstream()

  Revision 1.1.1.1  2002/09/25 19:00:35  shamus13
  Import of OmniEvents source tree from release 2.1.1

  Revision 1.8  2000/09/26 09:20:26  naderp
  STL Default parameters rework.
  Configurable checkpoint period.

  Revision 1.7  2000/09/04 05:05:22  naderp
  Fixed problem with triggering multiple checkpoints.

  Revision 1.6  2000/08/30 04:38:24  naderp
  Fix to prevent recorder thread from terminating.

  Revision 1.5  2000/08/30 00:57:29  naderp
  Fixed broken persist method exiting after first checkpoint.

  Revision 1.4  2000/03/06 13:19:58  naderp
  Moved port from global to factory persistency data.

  Revision 1.3  2000/03/06 04:15:09  naderp
  Removed internal dependency between factory and Naming Service.

  Revision 1.2  2000/03/02 04:20:09  naderp
  Initialising factory refernce in init().

  Revision 1.1  2000/03/02 02:00:25  naderp
  Re-open active logfile during re-start.

  Revision 1.0  1999/11/01 17:04:08  naderp
  Initial revision

*/

#include "omniEventsLog.h"

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>

#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif

#ifdef HAVE_FCNTL_H
#  include <fcntl.h>
#endif

#if defined(__VMS) && __CRTL_VER < 70000000
#  include <omniVMS/unlink.hxx>
#endif

#ifdef __WIN32__
#  include <io.h>
#  include <winbase.h>
#  define stat(x,y) _stat(x,y)
#  define unlink(x) _unlink(x)
#endif // __WIN32__

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#ifdef HAVE_LIBC_H
#  include <libc.h>
#endif

#ifdef HAVE_SYS_PARAM_H
#  include <sys/param.h>
#endif

#include <errno.h>
#include <time.h>
#include "gethostname.h"
#include "oep_global.h"

#include "EventChannelFactory.h"

//
// Set flags for use in calls to omniEventsLog::openOfstream()
//

#if defined(HAVE_FSTREAM_OPEN)
#  define FLAG_TRUNCATE ios::trunc
#  define FLAG_APPEND   ios::app
#  define FLAG_SYNC     0
#elif defined(HAVE_FSTREAM_ATTACH)
#  if defined(__WIN32__)
#    define FLAG_SYNC 0
#  elif defined(O_SYNC)
#    define FLAG_SYNC O_SYNC
#  else
#    define FLAG_SYNC O_FSYNC // FreeBSD 3.2 does not have O_SYNC???
#  endif
#  define FLAG_TRUNCATE O_CREAT|O_TRUNC
#  define FLAG_APPEND   O_APPEND
#else
#  error "Can't open a file without ofstream::open() or ofstream::attach()"
#endif

//
// Minimum idle period before we take a checkpoint (15 mins)
// (Set OMNIEVENTS_LOG_CHECKPOINT_PERIOD in include/config.h.)
//

#ifndef OMNIEVENTS_LOG_CHECKPOINT_PERIOD
#  define DEFAULT_IDLE_TIME_BTW_CHKPT  (15*60)
#else
#  define DEFAULT_IDLE_TIME_BTW_CHKPT OMNIEVENTS_LOG_CHECKPOINT_PERIOD 
#endif

//
// Append ';' to VMS filenames to force the latest version.
//

#ifdef __VMS
#  define VMS_SEMICOLON ";"
#else
#  define VMS_SEMICOLON
#endif

#define DB(l,x) ((omniORB::traceLevel >= l) && (cerr << x << endl))

extern int yyparse();
extern int yydebug;
extern FILE *yyin;

namespace OmniEvents {

//------------------------------------------------------------------------
//           timestamp Implementation
//------------------------------------------------------------------------
// This class can be used to generate timestamps.  The t() method normally
// returns a timestamp string, but if the same timestamp (to the nearest
// second) would be returned as last time then an empty string is returned
// instead.
//

class timestamp {
  char str[29];
public:
  timestamp(void) {
    str[0] = '\n';
    str[1] = str[28] = '\0';
  }
  char* t(void) {
    time_t t = time(NULL);
    char *p = ctime(&t);
    if (strncmp(p, &str[1], 24) == 0) {
      return &str[28];
    }
    strncpy(&str[1], p, 24);
    str[25] = ':';
    str[26] = '\n';
    str[27] = '\n';
    return str;
  }
};

timestamp ts;

//------------------------------------------------------------------------
//           omniEvents Log Implementation
//------------------------------------------------------------------------

omniEventsLog *omniEventsLog::theLog = NULL;

omniEventsLog::omniEventsLog(int p, const char* logdir) :
  port(p),
  logfile(),
  firstTime(0),
  active(NULL),
  backup(NULL),
  checkpt(NULL),
  recorder(NULL),
  factory(NULL),
  checkpointNeeded(1),
  _lock()
{
  omniEventsLog::theLog = this;
  
  initializeFileNames(logdir);

#ifdef __WIN32__
  struct _stat sb;
#else
  struct stat sb;
#endif

  if (port != 0)
  {

    //
    // Starting for the first time - make sure log file doesn't exist, and
    // for safety, that there is no backup file either.

    firstTime = 1;

    if (stat(active,&sb) == 0)
    {
      cerr << ts.t() << 
        "Error: persistency log file '" << active << "' exists.\n"
        " Run without the -s option to recover the"
        " server's state, or remove the file\n"
        " (and any backup) and then run with -s to"
        " create a new persistency log." << endl;
      exit(1);
    }
    if (stat(backup,&sb) == 0)
    {
      cerr << ts.t() <<
        "Error: backup file '" << backup << "' exists.\n"
        " Rename it to '" << active << "'\n"
        " and then run without the -s option to"
        " recover the server's state, or delete\n"
        " it and then run with -s to create a new persistency log." << endl;
      exit(1);
    }
  }
  else
  {

    //
    // Restart - parse log file.

    firstTime = 0;

    FILE *file = fopen(active, "r");

    if (!file)
    {
      cerr << ts.t() << "Error: cannot open persistency log file '" << active
           << "'." << endl;

      if (stat(backup,&sb) == 0) 
      {
        cerr <<
         " Backup file '" << backup << "' exists.\n"
         " Either rename it to '" << active << "' to\n"
         " recover the server's state, or delete it and"
         " then run with the -s option to\n"
         " create a new persistency log." <<endl;
      }
      else
      {
        cerr <<
         " Use the -s option the first time you run the server, to create the\n"
         " persistency log file." << endl;
      }
      exit(1);
    }

    yyin = file;
    oep_global = new OEP_GlobalData();
    int result = yyparse();
    if (result == 1)
    {
      cerr << ts.t() << "Error: parsing log file '" << active << "'." << endl;
      exit(1);
    }
    fclose(file);
    cerr << ts.t() << "Read log file successfully." << endl;

    //
    // Set the server port number
    OEP_cfps *fdata = oep_global->getFactory();
    port = fdata->getPort();
  }
}

void
omniEventsLog::init(EventChannelFactory_i *&f)
{
  factory = f;

  try
  {
    if (firstTime)
    {
      //
      // starting for the first time - create an initial log file.
      cerr << ts.t() << "Starting omniEvents on port "
           << port << endl;

      openOfstream(logfile,active,FLAG_TRUNCATE|FLAG_SYNC);
      _lock.lock();
      output(logfile);
      _lock.unlock();

      checkpointNeeded = 1;
      cerr << ts.t() << "Wrote initial log file.\n" << endl;
    }
    else
    {
      openOfstream(logfile,active,FLAG_APPEND);

      //
      // Get the persisted factory data
      OEP_cfps fdata =*oep_global->getFactory();

      //
      // Create the default factory
      factory =new EventChannelFactory_i(fdata.getPort(),fdata.getChannels());
      f = factory;
    }
  }
  catch (IOError& ex)
  {
    cerr << ts.t() << "Error: cannot open" << (firstTime?" initial":"")
         << " log file '" << active
         << "': " << endl;
    perror("");
    cerr << "\nYou can set the environment variable "
         << OMNIEVENTS_LOGDIR_ENV_VAR
         << " to specify the\ndirectory where the log files are kept.\n"
         << endl;
    logfile.close();
    unlink(active);
    exit(1);
  }

  //
  // Create a worker thread to checkpoint the log file at regular intervals.
  recorder = new omniEventsLogWorker(this,
                                     &omniEventsLog::checkpoint,
                                     omni_thread::PRIORITY_NORMAL);
  return;
}

void
omniEventsLog::persist()
{
  omniEventsLog *log = omniEventsLog::theLog;
  if(log && log->factory)
  {
    omni_mutex_lock l(log->_lock);
    log->output(log->logfile);
    log->logfile.flush();
    log->checkpointNeeded = 1;
  }
}

void
omniEventsLog::output(ostream& os)
{
  factory->output(os);
  os<<endl;
}

int
omniEventsLog::getPort()
{
  return port;
}

void
omniEventsLog::checkpoint(void)
{

  int idle_time_btw_chkpt;
  static int firstCheckPoint = 1;
  char *itbc = getenv("OMNIEVENTS_ITBC");
  if (itbc == NULL || sscanf(itbc,"%d",&idle_time_btw_chkpt) != 1)
  {
    idle_time_btw_chkpt = DEFAULT_IDLE_TIME_BTW_CHKPT;
  }

  omni_mutex mutex;
  omni_condition cond(&mutex);

  mutex.lock();
  while (1) {

    // Take an initial checkpoint the first time. All subsequent
    // checkpoints are conditionally tested on whether they are
    // needed or not.

    if (! firstCheckPoint)
    {
       unsigned long s, n;
       omni_thread::get_time(&s, &n, idle_time_btw_chkpt);
       cond.timedwait(s,n);

       _lock.lock();
       if (!checkpointNeeded)
       {
          _lock.unlock();
          continue;
       }
    }
    else
    {
       _lock.lock();
       firstCheckPoint = 0;
    }
  
    cerr << ts.t() << "Checkpointing Phase 1: Prepare." << endl;
  
    ofstream ckpf;
    int fd = -1;
  
    try
    {
      try
      {
        openOfstream(ckpf,checkpt,FLAG_TRUNCATE|FLAG_SYNC,&fd);
      }
      catch (IOError& ex)
      {
        cerr << ts.t() << "Error: cannot open checkpoint file '"
             << checkpt << "' for writing." << endl;
        throw;
      }
  
      output(ckpf);
  
      ckpf.close();
      if (!ckpf)
        throw IOError();
  
  // a bug in sparcworks C++ means that the fd doesn't get closed.
#if defined(__sunos__) && defined(__SUNPRO_CC) && __SUNPRO_CC < 0x500
      if (close(fd) < 0)
        throw IOError();
#endif
  
    } catch (IOError& ex) {
      cerr << ts.t() << flush;
      perror("I/O error writing checkpoint file");
      cerr << "Abandoning checkpoint" << endl;
      ckpf.close();
  // a bug in sparcworks C++ means that the fd doesn't get closed.
#if defined(__sunos__) && defined(__SUNPRO_CC) && __SUNPRO_CC < 0x500
      close(fd);
#endif
      unlink(checkpt);
      _lock.unlock();
      continue;
    }
  
    //
    // Now commit the checkpoint to become the active log.
    //
  
    cerr << ts.t() << "Checkpointing Phase 2: Commit." << endl;

  // a bug in sparcworks C++ means that the fd doesn't get closed.
#if defined(__sunos__) && defined(__SUNPRO_CC) && __SUNPRO_CC < 0x500
    close(logfile.rdbuf()->fd());
#endif
  
    logfile.close();
  
    unlink(backup);
  
#if defined(__WIN32__)
    if (rename(active, backup) != 0) {
#elif defined(__VMS)
    if (rename(active, backup) < 0) {
#else
    if (link(active,backup) < 0) {
#endif
      // Failure here leaves old active and checkpoint file.
      cerr << ts.t() << "Error: failed to link backup file '" << backup
           << "' to old log file '" << active << "'." << endl;
      exit(1);
    }
  
#if !defined( __VMS) && !defined(__WIN32__)
    if (unlink(active) < 0) {
      // Failure here leaves active and backup pointing to the same (old) file.
      cerr << ts.t() << "Error: failed to unlink old log file '" << active
           << "'." << endl;
      perror("");
      exit(1);
    }
#endif
  
#if defined(__WIN32__)
    if (rename(checkpt,active) != 0) {
#elif defined(__VMS)
    if (rename(checkpt,active) < 0) {
#else
    if (link(checkpt,active) < 0) {
#endif
      // Failure here leaves no active but backup points to the old file.
      cerr << ts.t() << "Error: failed to link log file '" << active
           << "' to checkpoint file '" << checkpt << "'." << endl;
      exit(1);
    }
  
#if !defined( __VMS) && !defined(__WIN32__)
    if (unlink(checkpt) < 0) {
      // Failure here leaves active and checkpoint pointing to the same file.
      cerr << ts.t() << "Error: failed to unlink checkpoint file '" << checkpt
           << "'." << endl;
      exit(1);
    }
#endif
  
    try
    {
      openOfstream(logfile,active,FLAG_APPEND|FLAG_SYNC,&fd);
    }
    catch (IOError& ex)
    {
      cerr << ts.t() << "Error: cannot open new log file '" << active
           << "' for writing." << endl;
      exit(1);
    }
  
    cerr << ts.t() << "Checkpointing completed." << endl;
  
    checkpointNeeded = 0;
    _lock.unlock();
  }
  mutex.unlock();
}


/** Sets the values of 'active', 'backup' and 'checkpt' file names.
 * The files are placed in the log file directory, that is specified by:
 *
 * - command line parameter -l,  OR ELSE
 * - OMNIEVENTS_LOGDIR_ENV_VAR env. var. (default `OMNIEVENTS_LOGDIR'), OR ELSE
 * - OMNIEVENTS_LOG_DEFAULT_LOCATION (set in include/config.h),  OR ELSE
 * - Unix: /var/omniEvents,  Win32: C:\TEMP,  VMS: []
 *
 * The current hostname is incorporated into the file names.
 */
void omniEventsLog::initializeFileNames(const char* logdir)
{
  if (!logdir)
      logdir = getenv(OMNIEVENTS_LOGDIR_ENV_VAR);
  if (!logdir)
  {
#if defined(OMNIEVENTS_LOG_DEFAULT_LOCATION)
    logdir = OMNIEVENTS_LOG_DEFAULT_LOCATION;
#elif defined(__WIN32__)
    logdir = "C:\\TEMP";
#elif defined(__VMS)
    logdir = "[]"; // What is a good default for VMS?
#else // Unix
    logdir = "/var/omniEvents";
#endif
  }

  const char* logname ="omnievents-";
  char hostname[MAXHOSTNAMELEN];
  if (0!=gethostname(hostname,MAXHOSTNAMELEN))
  {
    cerr << ts.t() << "Error: cannot get the name of this host." << endl;
    exit(1);
  }
  const char* sep ="";

#if defined(__WIN32__)
  sep="\\";
#elif defined(__VMS)
  char last( logdir[strlen(logdir)-1] );
  if (last != ':' && last != ']')
  {
    cerr << ts.t() << "Error: " << OMNIEVENTS_LOGDIR_ENV_VAR << " (" << logdir
         << ") is not a directory name." << endl;
    exit(1);
  }
#else // Unix
  if (logdir[0] != '/')
  {
    cerr << ts.t() << "Error: " << OMNIEVENTS_LOGDIR_ENV_VAR << " (" << logdir
         << ") is not an absolute path name." << endl;
    exit(1);
  }
  if (logdir[strlen(logdir)-1] != '/')
      sep="/";
#endif

  // VMS_SEMICOLON specifies latest version of the file on VMS
  // (essentially, we're saying we don't want to use VMS file versioning).

  setFilename(active, logdir,sep,logname,hostname,".log" VMS_SEMICOLON);
  setFilename(backup, logdir,sep,logname,hostname,".bak" VMS_SEMICOLON);
  setFilename(checkpt,logdir,sep,logname,hostname,".ckp" VMS_SEMICOLON);
}


/** Helper function that sets the value of the first parameter to the
 * concatenation of all the subsequent parameters.
 */
void
omniEventsLog::setFilename(
  char*& filename,     const char* logdir,   const char* sep,
  const char* logname, const char* hostname, const char* ext)
{
  size_t len=1+
    strlen(logdir)+strlen(sep)+strlen(logname)+strlen(hostname)+strlen(ext);
  filename=new char[len];
  sprintf(filename,"%s%s%s%s%s",logdir,sep,logname,hostname,ext);
}


/** Helper method that opens an output file stream using whatever method
 * is available. Available flags are:
 *
 * - FLAG_TRUNCATE
 * - FLAG_APPEND
 * - FLAG_SYNC
 *
 * @param s  reference to the ofstream object.
 * @param filename  the name of the file to open.
 * @param modeflags
 * @param fd  reference to a file descriptor. If used, this parameter is set
 *            to the fd that was opened, if any.
 */
void
omniEventsLog::openOfstream(
  ofstream& s, const char* filename, int flags, int* fd)
{
#if defined(HAVE_FSTREAM_OPEN)
#  ifdef FSTREAM_OPEN_PROT
      s.open(filename,ios::out|flags,0666);
#  else
      s.open(filename,ios::out|flags);
#  endif
      if (!s)
          throw IOError();

#elif defined(HAVE_FSTREAM_ATTACH)
#  ifdef __WIN32__
      int localFd = _open(filename, O_WRONLY | flags, _S_IWRITE);
#  else
      int localFd = open(filename, O_WRONLY | flags, 0666);
#  endif /* __WIN32__ */
      if (localFd < 0)
          throw IOError();
      if(fd)
          (*fd)=localFd;
      s.attach(localFd);
#endif
}


//------------------------------------------------------------------------
//           OmniEvents Log Worker Implementation
//------------------------------------------------------------------------
omniEventsLogWorker::omniEventsLogWorker(omniEventsLog *object,
                                       Method method,
                                       priority_t priority) :
   omni_thread(NULL, priority)
{

   DB(10, "omniEventsLogWorker : Constructor Start");

   _method = method;
   _object = object;

   start_undetached();

   DB(10, "omniEventsLogWorker : Constructor End");
}

void*
omniEventsLogWorker::run_undetached (void *) {

   DB(10, "omniEventsLogWorker : run_undetached Start");

   (_object->*_method)();

   DB(10, "omniEventsLogWorker : run_undetached End");

   return(0);
}

omniEventsLogWorker::~omniEventsLogWorker () {

   DB(10, "omniEventsLogWorker : Destructor Start");

   DB(10, "omniEventsLogWorker : Destructor End");
}


}; // end namespace OmniEvents

